;U W E  R E I S E R  Leipzig  april 2001
;led-display for ds1615
;osc XT, wdt off, power-up on

;register file deklaration
RTCC        equ     01h          ;rtcc-file
PC          equ     02h          ;program-counter

OPTION      equ     01h          ;option-file
STATUS      equ     03h          ;status-register

;i/o-register (ports)
PORTA       equ     05h          ;inputs
PORTB       equ     06h          ;inputs/outputs

;general purpose register
COUNT1      equ     10h          ;post-counter
COUNT2      equ     11h          ;1st bounce-time-counter
COUNT3      equ     12h          ;output-duration-counter
COUNT4      equ     13h          ;2nd bounce-time-counter
COUNT5      equ     14h          ;table-position
COUNT6      equ     15h          ;result for table-calculation
COUNT7      equ     16h          ;baud-counter
COUNT8      equ     17h          ;baudhalf-counter
COUNT9      equ     18h          ;bit-counter
LSB         equ     19h          ;lsb-display
MSB         equ     1Ah          ;msb-display
RESULT      equ     1Bh          ;read-result from ds1615
TIMEOUT     equ     1Ch          ;time-out for rs232

;variablen
C           equ     00h          ;carry-bit
W           equ     00h          ;result to w-file
U           equ     01h          ;result to u-file
Z           equ     02h          ;zero-bit
RX          equ     02h          ;rs232 recive
TX          equ     03h          ;rs232 transmit
RP0         equ     05h          ;bank0/bank1


;p r o g r a m
;initialisation
            MOVLW   0FFh         ;clear
            MOVWF   PORTA        ;port a &
            CLRF    PORTB        ;port b
            BCF     STATUS,RP0   ;select bank0
            CLRF    RTCC         ;clear rtcc + prescaler
            BSF     STATUS,RP0   ;select bank1
            MOVLW   00h          ;rb<7:0>
            MOVWF   PORTB        ;as outputs
            MOVLW   14h          ;ra2, ra4 as inputs
            MOVWF   PORTA        ;ra0, ra1, ra3 as outputs
            MOVLW   05h          ;select tmr0 prescaler
            MOVWF   OPTION       ;1:64 clock source intern
            BCF     STATUS,RP0   ;bank0


;power-on-self-test - yes or no (press button before power on!)
POSTSTART:  BTFSC   PORTA,4      ;power on self test ?
            GOTO    START        ;no
            CALL    BOUNCETIME   ;wait bounce-time
            BTFSC   PORTA,4      ;power on self test ?
            GOTO    START        ;no, error
            GOTO    POST         ;yes, to post


;table for lsb-values positve range (table-read)
POSTABLE:   MOVF    COUNT5,W     ;count5 to w-file
            ADDWF   PC,U         ;for pc-offset
            NOP
            RETLW   81h          ;0,0
            RETLW   80h          ;0,5
            RETLW   0E7h         ;1,0
            RETLW   0E6h         ;1,5
            RETLW   49h          ;2,0
            RETLW   48h          ;2,5
            RETLW   43h          ;3,0
            RETLW   42h          ;3,5
            RETLW   27h          ;4,0
            RETLW   26h          ;4,5
            RETLW   13h          ;5,0
            RETLW   12h          ;5,5
            RETLW   11h          ;6,0
            RETLW   10h          ;6,5
            RETLW   0C7h         ;7,0
            RETLW   0C6h         ;7,5
            RETLW   01h          ;8,0
            RETLW   00h          ;8,5
            RETLW   03h          ;9,0
            RETLW   02h          ;9,5


;table for lsb-values negative range (table read)
NEGTABLE:   MOVF    COUNT5,W     ;count5 to w-file
            ADDWF   PC,U         ;for pc-offset
            NOP
            RETLW   02h          ;9,5
            RETLW   03h          ;9,0
            RETLW   00h          ;8,5
            RETLW   01h          ;8,0
            RETLW   0C6h         ;7,5
            RETLW   0C7h         ;7,0
            RETLW   10h          ;6,5
            RETLW   11h          ;6,0
            RETLW   12h          ;5,5
            RETLW   13h          ;5,0
            RETLW   26h          ;4,5
            RETLW   27h          ;4,0
            RETLW   42h          ;3,5
            RETLW   43h          ;3,0
            RETLW   48h          ;2,5
            RETLW   49h          ;2,0
            RETLW   0E6h         ;1,5
            RETLW   0E7h         ;1,0
            RETLW   80h          ;0,5
            RETLW   81h          ;0,0


;subroutine power-on-self-test first part
POST:       MOVLW   1Fh          ;time-counter for
            MOVWF   COUNT1       ;post-value (-88,8)
POST1:      BSF     PORTA,0      ;t1 off (msb)
            MOVLW   00h          ;8,5
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
POST2:      BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    POST2
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   00h          ;-8 to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
POST3:      BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    POST3
            BSF     PORTA,0      ;t1 off (msb)
            MOVLW   01h          ;8,0
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
POST4:      BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    POST4
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   00h          ;-8 to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
POST5:      BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    POST5
            DECFSZ  COUNT1,U
            GOTO    POST1


;subroutine for first current-temperature after post
            BCF     PORTA,TX     ;55h request-command
            CALL    BAUD1        ;start-bit
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 0
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 1
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 2
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 3
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 4
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 5
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 6
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit


;test ds1615 recording (mip-bit)
;read page 00h, register 0014h (status register)
MIPBIT:     BCF     PORTA,TX     ;33h read-command
            CALL    BAUD1        ;start-bit
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 0
            CALL    BAUD1        ;bit 1
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 2
            CALL    BAUD1        ;bit 3
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 4
            CALL    BAUD1        ;bit 5
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 6
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

            BCF     PORTA,TX     ;00h msb register 0014h
            CALL    BAUD1        ;start-bit
            CALL    BAUD1        ;bit 0
            CALL    BAUD1        ;bit 1
            CALL    BAUD1        ;bit 2
            CALL    BAUD1        ;bit 3
            CALL    BAUD1        ;bit 4
            CALL    BAUD1        ;bit 5
            CALL    BAUD1        ;bit 6
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

            BCF     PORTA,TX     ;14h lsb register 0014h
            CALL    BAUD1        ;start-bit
            CALL    BAUD1        ;bit 0
            CALL    BAUD1        ;bit 1
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 2
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 3
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 4
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 5
            CALL    BAUD1        ;bit 6
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

;receive answer
            MOVLW   08h          ;value for
            MOVWF   COUNT9       ;bit-counter
            MOVLW   1Fh          ;value for
            MOVWF   TIMEOUT      ;time-out

W1:         DECFSZ  TIMEOUT,U    ;time-out ?
            GOTO    W2           ;no, next
            GOTO    FAULTW       ;yes, read-error rs232
W2:         BTFSS   PORTA,RX     ;test start-bit
            GOTO    W3           ;yes, start detect
            GOTO    W1           ;no, wait for start-bit
W3:         CALL    BAUDHALF     ;wait bounce
            BTFSC   PORTA,RX     ;test start-bit again
            GOTO    START        ;no, error
W4:         CALL    BAUD1        ;yes, wait for data-bit
            BTFSC   PORTA,RX     ;test data-bit
            CALL    SETBITW      ;data high
            CALL    CLRBITW      ;data low
W5:         RRF     RESULT,U     ;store information
            DECFSZ  COUNT9,U     ;test 8th bit
            GOTO    W4           ;no, next bit
            NOP                  ;yes,


;result mip-analysis
            BTFSS   RESULT,5     ;mip-bit high?
            GOTO    NOMIP        ;no,  show L,8
            NOP                  ;yes, show H,8


;yes mip - show H,8 second part power-on-self-test
MIP:        MOVLW   1Fh          ;time-counter for
            MOVWF   COUNT1       ;mip-value
MIP1:       BSF     PORTA,0      ;t1 off (msb)
            MOVLW   24h          ;H,5
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
MIP2:       BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    MIP2
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
MIP3:       BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    MIP3
            BSF     PORTA,0      ;t1 off (msb)
            MOVLW   25h          ;H,0
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
MIP4:       BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    MIP4
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
MIP5:       BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    MIP5
            DECFSZ  COUNT1,U
            GOTO    MIP1
            GOTO    START1       ;first run


;no mip - show L,8 second part power-on-self-test
NOMIP:      MOVLW   1Fh          ;time-counter for
            MOVWF   COUNT1       ;mip-value
NOMIP1:     BSF     PORTA,0      ;t1 off (msb)
            MOVLW   0B8h         ;L,5
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
NOMIP2:     BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    NOMIP2
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
NOMIP3:     BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    NOMIP3
            BSF     PORTA,0      ;t1 off (msb)
            MOVLW   0B9h         ;L,0
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
NOMIP4:     BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    NOMIP4
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
NOMIP5:     BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    NOMIP5
            DECFSZ  COUNT1,U
            GOTO    NOMIP1
            GOTO    START1       ;first run


;mainprogram
;first or next read- and display-cycle, yes or no?
START:      MOVLW   0FFh         ;clear
            MOVWF   PORTA        ;display
            MOVWF   PORTB
            BTFSC   PORTA,4      ;next cycle ?
            GOTO    START        ;no

;subroutine for first current-temperature after new pressed bottom
            BCF     PORTA,TX     ;55h request-command
            CALL    BAUD1        ;start-bit
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 0
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 1
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 2
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 3
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 4
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 5
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 6
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

            CALL    BOUNCETIME   ;wait bounce-time
            BTFSC   PORTA,4      ;next cycle ?
            GOTO    START        ;no, error


;read page 00h, register 0011h (current temperature)
            BCF     PORTA,TX     ;33h read-command
            CALL    BAUD1        ;start-bit
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 0
            CALL    BAUD1        ;bit 1
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 2
            CALL    BAUD1        ;bit 3
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 4
            CALL    BAUD1        ;bit 5
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 6
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

            BCF     PORTA,TX     ;00h msb register 0011h
            CALL    BAUD1        ;start-bit
            CALL    BAUD1        ;bit 0
            CALL    BAUD1        ;bit 1
            CALL    BAUD1        ;bit 2
            CALL    BAUD1        ;bit 3
            CALL    BAUD1        ;bit 4
            CALL    BAUD1        ;bit 5
            CALL    BAUD1        ;bit 6
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

            BCF     PORTA,TX     ;11h lsb register 0011h
            CALL    BAUD1        ;start-bit
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 0
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 1
            CALL    BAUD1        ;bit 2
            CALL    BAUD1        ;bit 3
            BSF     PORTA,TX
            CALL    BAUD1        ;bit 4
            BCF     PORTA,TX
            CALL    BAUD1        ;bit 5
            CALL    BAUD1        ;bit 6
            CALL    BAUD1        ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD1        ;stop-bit

;receive answer
            MOVLW   08h          ;value for
            MOVWF   COUNT9       ;bit-counter
            MOVLW   1Fh          ;value for
            MOVWF   TIMEOUT      ;time-out

R1:         DECFSZ  TIMEOUT,U    ;time-out ?
            GOTO    R2           ;no, next
            GOTO    FAULTU       ;yes, read-error rs232
R2:         BTFSS   PORTA,RX     ;test start-bit
            GOTO    R3           ;yes, start detect
            GOTO    R1           ;no, wait for start-bit
R3:         CALL    BAUDHALF     ;wait bounce
            BTFSC   PORTA,RX     ;test start-bit again
            GOTO    START        ;no, error
R4:         CALL    BAUD1        ;yes, wait for data-bit
            BTFSC   PORTA,RX     ;test data-bit
            GOTO    SETBITU      ;data high
            GOTO    CLRBITU      ;data low
R5:         RRF     RESULT,U     ;store information
            DECFSZ  COUNT9,U     ;test 8th bit
            GOTO    R4           ;no, next bit
            NOP                  ;yes, end
            GOTO    ALLOC        ;result-allocation
            

;request current temperature register in ds1615
START1:     BCF     PORTA,TX     ;55h request-command
            CALL    BAUD         ;start-bit
            BSF     PORTA,TX
            CALL    BAUD         ;bit 0
            BCF     PORTA,TX
            CALL    BAUD         ;bit 1
            BSF     PORTA,TX
            CALL    BAUD         ;bit 2
            BCF     PORTA,TX
            CALL    BAUD         ;bit 3
            BSF     PORTA,TX
            CALL    BAUD         ;bit 4
            BCF     PORTA,TX
            CALL    BAUD         ;bit 5
            BSF     PORTA,TX
            CALL    BAUD         ;bit 6
            BCF     PORTA,TX
            CALL    BAUD         ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD         ;stop-bit

  
;read page 00h, register 0011h (current temperature)
            BCF     PORTA,TX     ;33h read-command
            CALL    BAUD         ;start-bit
            BSF     PORTA,TX
            CALL    BAUD         ;bit 0
            CALL    BAUD         ;bit 1
            BCF     PORTA,TX
            CALL    BAUD         ;bit 2
            CALL    BAUD         ;bit 3
            BSF     PORTA,TX
            CALL    BAUD         ;bit 4
            CALL    BAUD         ;bit 5
            BCF     PORTA,TX
            CALL    BAUD         ;bit 6
            CALL    BAUD         ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD         ;stop-bit

            BCF     PORTA,TX     ;00h msb register 0011h
            CALL    BAUD         ;start-bit
            CALL    BAUD         ;bit 0
            CALL    BAUD         ;bit 1
            CALL    BAUD         ;bit 2
            CALL    BAUD         ;bit 3
            CALL    BAUD         ;bit 4
            CALL    BAUD         ;bit 5
            CALL    BAUD         ;bit 6
            CALL    BAUD         ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD         ;stop-bit

            BCF     PORTA,TX     ;11h lsb register 0011h
            CALL    BAUD         ;start-bit
            BSF     PORTA,TX
            CALL    BAUD         ;bit 0
            BCF     PORTA,TX
            CALL    BAUD         ;bit 1
            CALL    BAUD         ;bit 2
            CALL    BAUD         ;bit 3
            BSF     PORTA,TX
            CALL    BAUD         ;bit 4
            BCF     PORTA,TX
            CALL    BAUD         ;bit 5
            CALL    BAUD         ;bit 6
            CALL    BAUD         ;bit 7
            BSF     PORTA,TX     ;high for
            CALL    BAUD         ;stop-bit

 ;receive answer
            MOVLW   08h          ;value for
            MOVWF   COUNT9       ;bit-counter
            MOVLW   1Fh          ;value for
            MOVWF   TIMEOUT      ;time-out

U1:         DECFSZ  TIMEOUT,U    ;time-out ?
            GOTO    U2           ;no, next
            GOTO    FAULTU       ;yes, read-error rs232
U2:         BTFSS   PORTA,RX     ;test start-bit
            GOTO    U3           ;yes, start detect
            GOTO    U1           ;no, wait for start-bit
U3:         CALL    BAUDHALF     ;wait bounce
            BTFSC   PORTA,RX     ;test start-bit again
            GOTO    START        ;no, error
U4:         CALL    BAUD         ;yes, wait for data-bit
            BTFSC   PORTA,RX     ;test data-bit
            GOTO    SETBITU      ;data high
            GOTO    CLRBITU      ;data low
U5:         RRF     RESULT,U     ;store information
            DECFSZ  COUNT9,U     ;test 8th bit
            GOTO    U4           ;no, next bit
            NOP                  ;yes, end


;display allocation
ALLOC:      MOVLW   00h          ;check
            SUBWF   RESULT,W     ;result
            BTFSC   STATUS,Z     ;range
            GOTO    N400         ;result = -40,0C

            MOVLW   15h          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    N3XX         ;result <= -30,0C

            MOVLW   29h          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    N2XX         ;result <= -20,0C

            MOVLW   3Dh          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    N1XX         ;result <= -10,0C

            MOVLW   50h          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    N0XX         ;result <= -0,5C

            MOVLW   64h          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P0XX         ;result <= 9,5C

            MOVLW   78h          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P1XX         ;result <= 19,5C

            MOVLW   8Ch          ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P2XX         ;result <= 29,5C

            MOVLW   0A0h         ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P3XX         ;result <= 39,5C

            MOVLW   0B4h         ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P4XX         ;result <= 49,5C

            MOVLW   0C8h         ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P5XX         ;result <= 59,5C

            MOVLW   0DCh         ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P6XX         ;result <= 69,5C

            MOVLW   0F0h         ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P7XX         ;result <= 79,5C

            MOVLW   0FBh         ;check
            SUBWF   RESULT,W     ;result
            BTFSS   STATUS,C     ;range
            GOTO    P8XX         ;result <= 85,0C

            GOTO    START        ;wrong result

N400:       MOVLW   26h          ;-4
            MOVWF   MSB          ;to msb
            MOVLW   81h          ;0,0
            MOVWF   LSB          ;to lsb
            GOTO    DSP

N3XX:       MOVLW   42h          ;-3
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   01h
            MOVWF   COUNT6       ;result-value
X1:         MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X2           ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X1
X2:         CALL    NEGTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

N2XX:       MOVLW   48h          ;-2
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   15h
            MOVWF   COUNT6       ;result-value
X3:         MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X4           ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X3
X4:         CALL    NEGTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

N1XX:       MOVLW   0E6h         ;-1
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   29h
            MOVWF   COUNT6       ;result-value
X5:         MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X6           ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X5
X6:         CALL    NEGTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

N0XX:       MOVLW   7Fh          ;-
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   3Dh
            MOVWF   COUNT6       ;result-value
X7:         MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X8           ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X7
X8:         CALL    NEGTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P0XX:       MOVLW   0FFh         ;blank
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   50h
            MOVWF   COUNT6       ;result-value
X9:         MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X10          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X9
X10:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P1XX:       MOVLW   0E7h         ;1
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   64h
            MOVWF   COUNT6       ;result-value
X11:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X12          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X11
X12:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P2XX:       MOVLW   49h          ;2
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   78h
            MOVWF   COUNT6       ;result-value
X13:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X14          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X13
X14:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P3XX:       MOVLW   43h          ;3
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   8Ch
            MOVWF   COUNT6       ;result-value
X15:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X16          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X15
X16:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P4XX:       MOVLW   27h          ;4
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   0A0h
            MOVWF   COUNT6       ;result-value
X17:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X18          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X17
X18:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P5XX:       MOVLW   13h          ;5
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   0B4h
            MOVWF   COUNT6       ;result-value
X19:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X20          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X19
X20:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P6XX:       MOVLW   11h          ;6
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   0C8h
            MOVWF   COUNT6       ;result-value
X21:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X22          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X21
X22:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P7XX:       MOVLW   0C7h         ;7
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   0DCh
            MOVWF   COUNT6       ;result-value
X23:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X24          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X23
X24:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP

P8XX:       MOVLW   01h          ;8
            MOVWF   MSB          ;to msb
            MOVLW   01h
            MOVWF   COUNT5       ;table-position
            MOVLW   0F0h
            MOVWF   COUNT6       ;result-value
X25:        MOVF    COUNT6,W
            SUBWF   RESULT,W
            BTFSC   STATUS,Z     ;compare ?
            GOTO    X26          ;yes, new lsb
            INCF    COUNT6,U     ;no, next temp.-value
            INCF    COUNT5,U
            GOTO    X25
X26:        CALL    POSTABLE     ;value from table
            MOVWF   LSB          ;to lsb
            GOTO    DSP


;subroutine for display-service
DSP:        MOVLW   7Bh          ;output duration 2 sec
            MOVWF   COUNT3       ;with same value (3D = 1 sec)
DSP1:       BSF     PORTA,0      ;t1 off (msb)    (7B = 2 sec)
            MOVF    LSB,W        ;---------
            MOVWF   PORTB
            BCF     PORTA,1      ;t2 on (lsb)
DSP2:       BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    DSP2
            BSF     PORTA,1      ;t2 off (lsb)
            MOVF    MSB,W        ;--------- 61,05 Hz
            MOVWF   PORTB
            BCF     PORTA,0      ;t1 on (msb)
DSP3:       BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    DSP3
            DECFSZ  COUNT3,U     ;---------
            GOTO    DSP1
            BTFSC   PORTA,4      ;end ?
            GOTO    START        ;yes, waiting-loop
            GOTO    START1       ;no, next cycle


;subroutine data-rotation in result-register current temperature
SETBITU:    BSF     STATUS,C     ;data-bit high
            GOTO    U5
CLRBITU:    BCF     STATUS,C     ;data-bit low
            GOTO    U5


;subroutine data-rotation in result-register status register
SETBITW:    BSF     STATUS,C     ;data-bit high
            GOTO    W5
CLRBITW:    BCF     STATUS,C     ;data-bit low
            GOTO    W5


;subroutine for rs232-read-error current temperature - show E,8
FAULTU:     MOVLW   1Fh          ;time-counter for 10h= 524260
            MOVWF   COUNT1       ;mip-value        1Fh=1015780
FU1:        BSF     PORTA,0      ;t1 off (msb)
            MOVLW   18h          ;E,5
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
FU2:        BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    FU2
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
FU3:        BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    FU3
            BSF     PORTA,0      ;t1 off (msb)
            MOVLW   19h          ;E,0
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
FU4:        BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    FU4
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
FU5:        BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    FU5
            DECFSZ  COUNT1,U
            GOTO    FU1
            GOTO    START        ;start main-program


;subroutine for rs232-read-error status register - show E,8
FAULTW:     MOVLW   1Fh          ;time-counter for
            MOVWF   COUNT1       ;mip-value
FW1:        BSF     PORTA,0      ;t1 off (msb)
            MOVLW   18h          ;E,5
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
FW2:        BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    FW2
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
FW3:        BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    FW3
            BSF     PORTA,0      ;t1 off (msb)
            MOVLW   19h          ;E,0
            MOVWF   PORTB        ;to lsb
            BCF     PORTA,1      ;t2 on (lsb)
FW4:        BTFSS   RTCC,7       ;8187 s 1:64
            GOTO    FW4
            BSF     PORTA,1      ;t2 off (lsb)
            MOVLW   0FFh         ;blank to
            MOVWF   PORTB        ;msb
            BCF     PORTA,0      ;t1 on (msb)
FW5:        BTFSC   RTCC,7       ;8192 s 1:64
            GOTO    FW5
            DECFSZ  COUNT1,U
            GOTO    FW1
            GOTO    MIPBIT       ;new post


;subroutine bouncetime for switch test
BOUNCETIME: MOVLW   0FFh         ;FFh = 196,350 ms
            MOVWF   COUNT2
            MOVLW   0FFh
            MOVWF   COUNT4
LOOP1:      DECFSZ  COUNT4,U
            GOTO    LOOP1
            DECFSZ  COUNT2,U
            GOTO    LOOP1
            RETURN


;subroutine boud-rate 103 s with display refresh for start1
BAUD:       MOVLW   1Eh          ;= 97 s
            MOVWF   COUNT7       ;+ 2 s call
            NOP                  ;1 s delay
            NOP                  ;1 s delay
            BSF     PORTA,0      ;refresh
            MOVF    LSB,W        ;  lsb
            MOVWF   PORTB        ; digit
            BCF     PORTA,1      ;displays
LOOPA:      DECFSZ  COUNT7,U     ;+ 2 s return
            GOTO    LOOPA
            RETURN


;subroutine boud-rate  104 s for post and start
BAUD1:      MOVLW   20h          ;= 100 s
            MOVWF   COUNT7       ;+ 2 call
LOOP2:      DECFSZ  COUNT7,U     ;+ 2 return
            GOTO    LOOP2
            RETURN


;subroutine boud-half-rate
BAUDHALF:   MOVLW   0Ah          ;= 50 s
            MOVWF   COUNT8       ;+ 2s call
LOOP3:      DECFSZ  COUNT8,U     ;+ 2s return
            GOTO    LOOP3
            RETURN


            END